const Clutter = imports.gi.Clutter;
const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Lang = imports.lang;
const Meta = imports.gi.Meta;
const St = imports.gi.St;
const Shell = imports.gi.Shell;
const Signals = imports.signals;
const FileUtils = imports.misc.fileUtils;
const Main = imports.ui.main;
const ModalDialog = imports.ui.modalDialog;
const ShellEntry = imports.ui.shellEntry;
const Tweener = imports.ui.tweener;
const Util = imports.misc.util;
const Utils = imports.misc.extensionUtils.getCurrentExtension().imports.utils;
const ScreenShot = imports.misc.extensionUtils.getCurrentExtension().imports.screenshot;
const Extension = imports.misc.extensionUtils.getCurrentExtension();

const DIALOG_GROW_TIME = 0.1;

const HelpMessage = '\nHelp for aBox :\n\n' + 'aBox add tons of web apps to your desktop\n'
					 + ' to search : \'engine1,engine2,engine3,...:search terms or expression\'\n'
					 + ' to launch a web app : \'u:adr\' or \':shortcut\'\n\n' 
					 + '"h": help\n' 
					 + '"ex": show examples\n'
					 + '"k": xkill\n' 
					 + '"r": restart the shell\n'
					 + '"t": launch a terminal\n'
					 + '"e": edit json shortcuts file (USE WITH CAUTION)\n'
					 + '"sh": show actual shortcuts\n\n'
					 + '"d:text" write a text dump\n'
					 + '"ds" show text dumps\n'
					 + '"de" edit text dumps\n\n'
					 + '"u:url" open url as a web app : skip the http://www part, and the .com (but not .org)\n'
					 + '  for instance, "u:liberation:lbr" will open a web application for www.liberation.com\n'
					 + '  and set up "lbr" as a short cut\n'
					 + '"s:shrt" or ":shrt" open the url corresponing to shrt\n\n'
					 + '"shot:name" take a screenshot of a window "name" can be either contained in the title\n'
					 + '   or in the application name. The screenshot is saved in the picture directory.\n\n'
					 + '"engine:search terms" search engine for the search terms\n'
					 + ' available are : \n'
					 + '"g" : google, "gm" : google maps, "gn" : google news, "kd" : koders\n'
					 + '"fk" : flicker, "b" : bing, "bi" : bing images, "bv" : bing videos\n'
					 + '"bn" : bing news, "bm" : bing maps, "yt" : yoututbe, "wp" : wikipedia\n'
					 + '"wr" : wolfram (science, maths..), "imdb" : imdb\n'
					 + ' a leading "+" launch traditional browser : "+u:yahoo:yh" or "+gg,wp,gm:antalya"\n'
					 + ' searches with no query launches the home page of the site\n'
					 + ' several commands can be launched, using ";" : "s:yh;+gg,wp,gm,fk:berlin"\n';


const ExampleMessage = '\nExamples of queries :\n\n' 
					 + ' open an url - "u:address" :\n'
					 + ' "u:yahoo" "u:en.wikipedia.org" "u:google" \n\n'
					 + ' open an url and save a corresponding shortcut "u:address:shortcut" :\n'
					 + ' "u:yahoo:yh" , "u:en.wikipedia.org:wp" , "u:rdio:rd"\n\n'
					 + ' launch a command in a normal tabbed browser :\n' 
					 + ' "+u:google"\n\n' 
					 + ' open a shortcut defined using "u:address:shortcut" :\n'
					 + ' "s:shortcut" or ":shortcut" for instance ":yh"\n\n' 
					 + ' launch a search in normal tabbed browser mode "engine:search terms" :\n'
					 + ' search for "europe" in google and bing :\n'
					 + ' "+g,b:europe"\n\n'
					 + ' search for "samsung galaxy" in google news, bing news, wikipedia, \n'
					 + ' google, bing and youtube :\n'
					 + ' "+gn,bn,wp,g,b,yt:samsung galaxy"\n\n' 
					 + ' multiple commands :\n'
					 + ' "+gn,bn:iphone;s:yh;u:deezer:dz"\n';

const ExtensionDir = Extension.dir;

let language = "en";
let abox_obj;
let screenshot_obj;
let searchProviders = [];
let jsonshortcut = [];
let jsondumps = [];

const key_bindings = {
	'show-abox': function() {
		abox_obj.open();
	}
};

const mySettings = Utils.getSettings();


const aBoxClass = new Lang.Class({
    Name: 'aBoxClass',
    Extends: ModalDialog.ModalDialog,

    _init : function() {
        this.parent({ styleClass: 'abox-dialog' });
				
        let label = new St.Label({ style_class: 'abox-dialog-label',
                                   text: _("aBox (h for help) :") });

        this.contentLayout.add(label, { y_align: St.Align.START });

        let entry = new St.Entry({ style_class: 'abox-dialog-entry' });
        ShellEntry.addContextMenu(entry);

        entry.label_actor = label;

        this._entryText = entry.clutter_text;
        this.contentLayout.add(entry, { y_align: St.Align.START });
        this.setInitialKeyFocus(this._entryText);

        this._helpBox = new St.BoxLayout({ style_class: 'abox-dialog-help-box' });

        this.contentLayout.add(this._helpBox, { expand: true });

        this._commandText = false;

        this._helpMessage = new St.Label({ style_class: 'abox-dialog-help-label' });
        this._helpMessage.clutter_text.line_wrap = true;

        this._helpBox.add(this._helpMessage, { expand: true,
                                                 y_align: St.Align.MIDDLE,
                                                 y_fill: false });

        this._helpBox.hide();

        this._entryText.connect('key-press-event', Lang.bind(this, function(o, e) {
            let symbol = e.get_key_symbol();
            if (symbol == Clutter.Return || symbol == Clutter.KP_Enter) {
                this.popModal();
                if (e.get_state() & Clutter.ModifierType.CONTROL_MASK)
								{
									parseText(o.get_text());
                } else
								{
									parseText(o.get_text());
								}
                if (!this._commandText)
                    this.close();
                else {
                    if (!this.pushModal())
                        this.close();
                }
                return true;
            }
            if (symbol == Clutter.Escape) {
                this.close();
                return true;
            }
						return false;
				}));
    },

    _run : function(input) {
        let command = input;

        this._commandText = false;
				if (input) {
					try {
						Util.trySpawnCommandLine(command);
					} catch (e) {
						this._showText(e.message);
					}
				}
    },

    _showText : function(message) {
        this._commandText = true;

        this._helpMessage.set_text(message);

        if (!this._helpBox.visible) {
            let [helpBoxMinHeight, helpBoxNaturalHeight] = this._helpBox.get_preferred_height(-1);

            let parentActor = this._helpBox.get_parent();
            Tweener.addTween(parentActor,
                             { height: parentActor.height + helpBoxNaturalHeight,
                               time: DIALOG_GROW_TIME,
                               transition: 'easeOutQuad',
                               onComplete: Lang.bind(this,
                                                     function() {
                                                         parentActor.set_height(-1);
                                                         this._helpBox.show();
                                                     })
                             });
        }
    },

    open: function() {
        this._helpBox.hide();
        this._entryText.set_text('');
        this._commandText = false;
				this.parent();
    },
});
Signals.addSignalMethods(aBoxClass.prototype);


function init()
{
	initSearchProviders();
	parseDumps();
	parseShortCutHistory();
}

function enable()
{	
  for(key in key_bindings) {
		global.display.add_keybinding(key,
			mySettings,
			Meta.KeyBindingFlags.NONE,
			key_bindings[key]
		);
	}

	abox_obj = new aBoxClass();
	screenshot_obj = new ScreenShot.WindowScreenshot();
}

function disable()
{
	for(key in key_bindings) {
		global.display.remove_keybinding(key);
	}

	abox_obj = null;
}

function editDumps()
{
	let command = 'gedit ' + _dumpFile;
	abox_obj._run(command,false);
}


function showDumps()
{
		let i;
		let dumpMessage = 'dumps :\n\n'; 
		for (i = 0; i < jsondumps.length ; i++)
		{
			let jsonstring = jsondumps[i];

			if (jsonstring != null)
			{
				dumpMessage = dumpMessage + jsonstring.dump + "\n";	
			}
		}
		abox_obj._showText(dumpMessage);
}

function editShrtCuts()
{
	let command = 'gedit ' + _configFile;
	abox_obj._run(command,false);
}


function showShrtCuts()
{
		let i;
		let shrtCutMessage = 'shortcuts :\n\n'; 
		for (i = 0; i < jsonshortcut.length ; i++)
		{
			let jsonstring = jsonshortcut[i];

			if (jsonstring != null)
			{
				shrtCutMessage = shrtCutMessage + jsonstring.shrt + ' : ' +  jsonstring.addr + "\n";	
			}
		}
		abox_obj._showText(shrtCutMessage);
}

function help()
{
	abox_obj._showText(HelpMessage);
}

function showEx()
{
	abox_obj._showText(ExampleMessage);
}

function launchSearchForProvider(toks,sP,begin)
{
	let command;
	command = begin + sP.site;

	if (toks.length > 1)
	{
		if (sP.abr == 'wr')
		{	
			command = begin + sP.srchString + toks[1]; 
		} else
		{
			let query = toks[1].split(" ");

			if (query.length != 0)
			{
				let i;
				command = begin + sP.srchString; 

				for (i = 0; i < query.length - 1; i++)
				{
					command = command + query[i] + sP.smb;
				}

				command = command + query[query.length-1];
				command = command + sP.addon;
			}
		}
	}
	abox_obj._run(command,false);
}


function initSearchProviders()
{
	searchProviders.push({ abr : 'kd', srchString : 'http://www.koders.com/default.aspx?s=', site: 'http://www.koders.com' , addon:'\&submit=Search&la=*\&li=*' , smb:'+'});
	searchProviders.push({ abr : 'gn', srchString : 'http://www.google.com/search?hl=' + language + '\&gl=' + language + '\&tbm=nws\&q=', site: 'http://news.google.com' , addon:'' , smb:'+'});
	searchProviders.push({ abr : 'g', srchString : 'http://www.google.com/?q=', site: 'http://www.google.com' , addon:'' , smb:'+'});
	searchProviders.push({ abr : 'gm', srchString : 'http://maps.google.com/?q=', site: 'http://maps.google.com' , addon:'' , smb:'+'});
	searchProviders.push({ abr : 'imdb', srchString : 'http://www.imdb.com/find?q=', site: 'http://www.imdb.com' , addon:'' , smb:'+'});
	searchProviders.push({ abr : 'yt', srchString : 'http://www.youtube.com/results?search_query=', site: 'http://www.youtube.com' , addon:'' , smb:'+'});
	searchProviders.push({ abr : 'fk', srchString : 'http://www.flickr.com/search/?q=', site: 'http://www.flickr.com' , addon:'' , smb:'+'});
	searchProviders.push({ abr : 'b', srchString : 'http://www.bing.com/search/?q=', site: 'http://www.bing.com' , addon:'' , smb:'+'});
	searchProviders.push({ abr : 'bi', srchString : 'http://www.bing.com/images/search/?q=', site: 'http://www.bing.com/images' , addon:'' , smb:'+'});
	searchProviders.push({ abr : 'bv', srchString : 'http://www.bing.com/videos/search/?q=', site: 'http://www.bing.com/videos' , addon:'' , smb:'+'});
	searchProviders.push({ abr : 'bn', srchString : 'http://www.bing.com/news/search/?q=', site: 'http://www.bing.com/news' , addon:'' , smb:'+'});
	searchProviders.push({ abr : 'bm', srchString : 'http://www.bing.com/maps/default.aspx?q=', site: 'http://www.bing.com/maps' , addon:'' , smb:'+'});
	searchProviders.push({ abr : 'wp', srchString : 'http://' + language + '.wikipedia.org/wiki/Special:Search?search=', site: 'http://' + language + '.wikipedia.org' , addon:'' , smb:'+'});
	searchProviders.push({ abr : 'wr', srchString : 'http://www.wolframalpha.com/input/?i=' , site: 'http://www.wolframalpha.com' , addon:'' , smb:'+'}); 

}

let _configDir = GLib.get_user_config_dir() + "/gnome-shell-abox";
let _configFile = _configDir + "/gnome_shell_abox.json";
let _dumpFile = _configDir + "/gnome_shell_abox_dumps.json";
 
function parseDumps() {
	if (GLib.file_test(_dumpFile, GLib.FileTest.EXISTS)) {
		let filedata = null;
		try {
			let k;
			filedata = GLib.file_get_contents(_dumpFile, null, 0);
			let jsonstring = JSON.parse(filedata[1]);
			let jsonarray = jsonstring["dumps"]; 
			for (k = 0; k < jsonarray.length; k++)
			{
				jsondumps.push(jsonarray[k]);
			}
		}
		catch (e) {
			global.logError("aBox: Error reading dumps file " + _configFile + ", error = " + e);
		}
		finally {
			filedata = null;
		}
	}
}
 
function parseShortCutHistory() {
	if (GLib.file_test(_configFile, GLib.FileTest.EXISTS)) {
		let filedata = null;
		try {
			let k;
			filedata = GLib.file_get_contents(_configFile, null, 0);
			let jsonstring = JSON.parse(filedata[1]);
			let jsonarray = jsonstring["shortcuts"]; 
			for (k = 0; k < jsonarray.length; k++)
			{
				jsonshortcut.push(jsonarray[k]);
			}
		}
		catch (e) {
			global.logError("aBox: Error reading config file " + _configFile + ", error = " + e);
		}
		finally {
			filedata = null;
		}
	}
}

function writeDump(dmp) {
	let filedata = '{\n"dumps":\n[\n';
	let jsondata;

	if (GLib.file_test(_configDir, GLib.FileTest.EXISTS | GLib.FileTest.IS_DIR) == false &&
			GLib.mkdir_with_parents(_configDir, 0755) != 0) {
		global.logError("aBox: Failed to create configuration directory. Path = " +
				_configDir + ". Configuration and dumps will not be saved.");
	}

	try {
		let i;
		jsondata = { dump : dmp};
		jsondumps.push(jsondata);

		for (i = 0; i < jsondumps.length - 1; i++)
		{
			let jsonstring = jsondumps[i];

			if (jsonstring != null)
			{
				filedata = filedata + JSON.stringify(jsonstring, null, "  ") + ',\n';
			}
		}
		let jsonstring = jsondumps[jsondumps.length - 1];

		if (jsonstring != null)
		{
			filedata = filedata + JSON.stringify(jsonstring, null, "  ") + '\n';
		}

		filedata = filedata +']\n}\n';

		GLib.file_set_contents(_dumpFile, filedata, filedata.length);
	}
	catch (e) {
		global.logError("aBox: Error writing dump file = " + e);
	}
	finally {
		jsondata = null;
		filedata = null;
	}
}	


function saveShortCutHistory(shortcut,address) {
	let filedata = '{\n"shortcuts":\n[\n';
	let jsondata;

	if (GLib.file_test(_configDir, GLib.FileTest.EXISTS | GLib.FileTest.IS_DIR) == false &&
			GLib.mkdir_with_parents(_configDir, 0755) != 0) {
		global.logError("aBox: Failed to create configuration directory. Path = " +
				_configDir + ". Configuration and shortcuts will not be saved.");
	}

	try {
		let i;
		jsondata = { shrt : shortcut, addr : address};
		jsonshortcut.push(jsondata);

		for (i = 0; i < jsonshortcut.length - 1; i++)
		{
			let jsonstring = jsonshortcut[i];

			if (jsonstring != null)
			{
				filedata = filedata + JSON.stringify(jsonstring, null, "  ") + ',\n';
			}
		}
		let jsonstring = jsonshortcut[jsonshortcut.length - 1];

		if (jsonstring != null)
		{
			filedata = filedata + JSON.stringify(jsonstring, null, "  ") + '\n';
		}

		filedata = filedata +']\n}\n';

		GLib.file_set_contents(_configFile, filedata, filedata.length);
	}
	catch (e) {
		global.logError("aBox: Error writing shortcut file = " + e);
	}
	finally {
		jsondata = null;
		filedata = null;
	}
}	

function getAddressForShortCut(shortcut)
{
	let i;
	for (i = 0; i < jsonshortcut.length;i++)
	{
		let jsondata = jsonshortcut[i];

		if (jsondata.shrt == shortcut)
		{
			return jsondata.addr;
		}
	}
	return '';
}

function defineFunctionToExecute(text,begin)
{
	let j,k;
	let toks = text.split(":");
	let cmds = toks[0].split(",");


	for (j = 0; j < cmds.length ; j++)
	{
		let goon = true;
		let command;

		for (k = 0; (k < searchProviders.length) && (goon == true); k++)
		{
			let sP = searchProviders[k];
			if (cmds[j] == sP.abr)
			{
				launchSearchForProvider(toks,sP,begin);
				goon = false;
			}
		}

		if (goon == true)
		{
			if (toks.length == 1)
			{
				if (cmds[j] == 'r')
				{
					global.reexec_self();
				} else if (cmds[j] == 'h') /* commands */
				{
					help();
				} else if (cmds[j] == 'e') 
				{
					editShrtCuts();
				} else if (cmds[j] == 'ex') 
				{
					showEx();
				} else if (cmds[j] == 'sh') 
				{
					showShrtCuts();
				} else if (cmds[j] == 'k')
				{
					abox_obj._run('xkill',false);
				} else if (cmds[j] == 't')
				{
					command = 'gnome-terminal';
					abox_obj._run(command,false);
				} else if (cmds[j] == 'rjs') /* specific */
				{
					command = begin + 'http://www.roojs.com/seed/gir-1.2-gtk-3.0/gjs/';
					abox_obj._run(command,false);
				}	
			} else if (toks.length > 1)
			{
				if (cmds[j] == 'shot') 
				{
					try
					{
						screenshot_obj._prepareWindowScreenshot(toks[1]);
					}
					catch (e) {
						global.logError("aBox: Error taking the screenshot error = " + e);
					}

				} else if (cmds[j] == 's') /* web navigation */
				{
					let addr = getAddressForShortCut(toks[1]);

					if (addr != '')
					{
						command = begin + 'http://' + addr;
						abox_obj._run(command,false);
					}
				} else if (cmds[j] == 'u')
				{
					let addr;
					let domain = toks[1].split(".");

					if (domain.length == 1)
					{
						command = begin + 'http://' + domain[0] + '.com';
						addr =  domain[0] + '.com';
					}
					else
					{
						command = begin + 'http://' + toks[1];
						addr = toks[1];
					} 

					// eventually save the shortcut
					if (toks.length > 2)
					{
						saveShortCutHistory(toks[2],addr);
					}
					abox_obj._run(command,false);
				} 
			}
		}
	}
}

function parseText(request)
{
	let	begin = "google-chrome --app=";
	request = ";" + request + ";";
	let subrequestcmp = request.split(";");

	for (i = 0; (i < subrequestcmp.length - 1) && (i<10); i++)
	{
		let subrequest = subrequestcmp[i];

		if (subrequest.length != 0)
		{
			if (subrequest[0] == "+")
			{
				begin = "google-chrome ";
				defineFunctionToExecute(subrequest.substring(1,subrequest.length),begin);
			}
			else 
			{
				if (subrequest[0] == "d")
				{
					/* dump text */
					if (subrequest[1] == ":")
					{
						writeDump(subrequest.substring(2,subrequest.length));	
					}
					else if (subrequest[1] == "s")
					{
						showDumps();
					}
					else if (subrequest[1] == "e")
					{
						editDumps();
					}
				} 
				else
				{
					if (subrequest[0] == ":")
					{
						subrequest = 's'+ subrequest;
					}
					begin = "google-chrome --app=";
					defineFunctionToExecute(subrequest,begin);
				}
			}
		}
	}
}

