const Clutter   = imports.gi.Clutter;
const Lang      = imports.lang;
const Shell     = imports.gi.Shell;
const Mainloop  = imports.mainloop;
const GLib      = imports.gi.GLib;
const Gdk       = imports.gi.Gdk;
const St        = imports.gi.St;
const Main      = imports.ui.main;
const Util      = imports.misc.util;
const Tweener   = imports.ui.tweener;

function WindowScreenshot() {};

WindowScreenshot.prototype = {

		_close: function() {
    },

		_isStringIn: function(texttosearch,instring) {
		 let i;

		 for (i = 0; i < instring.length; i++)
		 {
			 if (i + texttosearch.length <= instring.length)
			 {
				 if (texttosearch == instring.substring(i,i + texttosearch.length))
				 {
						 return true;
				 }
			 }
		 }
		 return false;

	 },

    _prepareWindowScreenshot: function(textsearch) {
        this._close();

        let windows = Main.getWindowActorsForWorkspace(global.screen.get_active_workspace_index()); 
        let window  = null;

        for (let i = (windows.length - 1); i >= 0; i--) {

            let metaWindow = windows[i].get_meta_window();
						let title = metaWindow.title;
						let wmclass = metaWindow.get_wm_class();
            if (this._isStringIn(textsearch,title) ||  this._isStringIn(textsearch,wmclass)) {
                window = metaWindow;
								break;
            }
        }

        if (!window) {
            // Can this really happen? At least the root window should match.
            return;
        }

        if (window.has_focus()) {
            this._makeWindowScreenshot();
        } else {
            let tracker      = Shell.WindowTracker.get_default();
            let focusEventId = tracker.connect('notify::focus-app', Lang.bind(this, function() {
                // Without this timeout, we will get a memory access violation.
                let timeoutId = Mainloop.timeout_add(1, Lang.bind(this, function() {
                    this._makeWindowScreenshot();
                    Mainloop.source_remove(timeoutId);
                    return false;
                }));

                tracker.disconnect(focusEventId);
            }));

            Main.activateWindow(window)
        }
    },

    _makeWindowScreenshot: function () {
        let filename = this._getNewScreenshotFilename();

        let screenshot = new Shell.Screenshot();
        screenshot.screenshot_window(true, false, filename,
            Lang.bind(this, this._onScreenshotComplete, filename))
    },

    _onScreenshotComplete: function(obj, result, area, filename) {
        global.cancel_theme_sound(SHUTTER_NOTIFY_ID);
        global.play_theme_sound(SHUTTER_NOTIFY_ID, 'camera-shutter');
    },

    _getNewScreenshotFilename: function() {
        let date     = new Date();
        let filename = GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_PICTURES) + '/'
                     + 'screenshot-'
                     + date.getFullYear() + '-'
                     + this._padNum(date.getMonth() + 1) + '-'
                     + this._padNum(date.getDate()) + 'T'
                     + this._padNum(date.getHours()) + ':'
                     + this._padNum(date.getMinutes()) + ':'
                     + this._padNum(date.getSeconds())
                     + '.png';

        return filename;
    },

    _padNum: function(num) {
        return (num < 10 ? '0' + num : num);
    }
}

